import { createElement, useMemo } from '@wordpress/element'

import { useSelect } from '@wordpress/data'
import Checkbox from './Checkbox'
import Label from './Label'
import Counter from './Counter'

const HierarchicalItem = ({
	taxItem,
	showLabel,
	taxonomy,
	showCheckbox,
	showCounters,
	hierarchical,
	expandable,
	defaultExpanded,
	excludeTaxonomy,
	taxonomy_not_in,
	showItemsRendered,
}) => {
	const allCat = useSelect(
		(select) =>
			select('core').getEntityRecords('taxonomy', taxonomy, {
				per_page: -1,
			}) || []
	)

	const children = useMemo(() => {
		return allCat.filter((cat) => cat.parent === taxItem.id)
	}, [taxItem.id])

	const showTaxLabels =
		(taxonomy === 'product_brands' && showLabel) ||
		taxonomy !== 'product_brands'

	return (
		<li className="ct-filter-item">
			<div className="ct-filter-item-inner">
				<a href="#">
					<Checkbox showCheckbox={showCheckbox} />

					{taxonomy === 'product_brands' &&
					taxItem?.logo?.url &&
					showItemsRendered ? (
						<div className="ct-media-container ct-product-brand">
							<img src={taxItem.logo.url} alt={taxItem.name} />
						</div>
					) : null}

					<Label
						showLabel={showTaxLabels}
						label={taxItem.name}
						count={taxItem.count}
						withCount={showCounters && expandable && hierarchical}
					/>

					<Counter
						count={taxItem.count}
						showCounters={
							showCounters && (!expandable || !hierarchical)
						}
					/>
				</a>
				{children.length && hierarchical && expandable ? (
					<span className="ct-expandable-trigger">
						<svg
							class="ct-icon"
							width="10"
							height="10"
							viewBox="0 0 25 25">
							<path d="M.207 17.829 12.511 5.525l1.768 1.768L1.975 19.596z"></path>
							<path d="m10.721 7.243 1.768-1.768L24.793 17.78l-1.768 1.767z"></path>
						</svg>
					</span>
				) : null}
			</div>
			{children.length && hierarchical && defaultExpanded ? (
				<ul className="ct-filter-children">
					{children.map((item) => {
						if (
							taxonomy_not_in.includes(item.id) &&
							excludeTaxonomy
						) {
							return null
						}

						return (
							<HierarchicalItem
								key={item.id}
								taxonomy={taxonomy}
								taxItem={item}
								showCheckbox={showCheckbox}
								showCounters={showCounters}
								hierarchical={hierarchical}
								expandable={expandable}
								taxonomy_not_in={taxonomy_not_in}
								excludeTaxonomy={excludeTaxonomy}
								showItemsRendered={showItemsRendered}
								showLabel={showTaxLabels}
							/>
						)
					})}
				</ul>
			) : null}
		</li>
	)
}

export default HierarchicalItem
